/*
 * THERMOClick Driver
 * 
 * Version 1.0 - using Software SPI through Bit-Banging
 * - Initial version coded Stephen Cardinale
 *
 * Version 1.1
 * - Revision to include Hardware SPI
 *
 *  Version 2.0 :
 *  - General refactoring and code cleanup.
 *  - Removed Bit-Banging Code (software SPI).
 *  - Integration of the new namespaces and new organization.
 * 
 * References Needed :
 * Microsoft.Spot.Hardware
 * Microsoft.spot.Native
 * MikroBusNet
 * mscorlib
 * Notes - Add the conditional compilation statement USE_SPI for SPI or omit for Bit-Banging.
 * 
 * Copyright 2014 Mikrobus.Net
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions and limitations under the License. 
 */

using System;

namespace MBN.Modules
{
    public partial class ThermoClick
    {
        /// <summary>
        /// A class to hold the Sensor Data as read from the ThermoClick.
        /// </summary>
        internal class ThermoClickSensorData
        {
            private readonly byte[] _sample = new byte[4];

            /// <summary>
            /// Initializes a new instance of the  <see cref="ThermoClickSensorData"/> class.
            /// </summary>
            /// <param name="sample">Must be 4 bytes in length</param>
            public ThermoClickSensorData(byte[] sample)
            {
                IsSensorDataValid = (sample.Length == 4) &
                                    !(sample[0] == 0xff && sample[1] == 0xff && sample[2] == 0xff && sample[3] == 0xff);
                Array.Copy(sample, _sample, _sample.Length);
            }

            /// <summary>
            /// True for if the data was pass was the correct length. Does not indicate if there were faults with the sensor.
            /// </summary>
            internal bool IsSensorDataValid { get; private set; }

            /// <summary>
            /// Thermocouple temperature in degrees Celsius.
            /// </summary>
            /// <returns>
            /// The temperature of the Thermocouple attached to the ThermoClick, or Negative Infinity if the ThermoClick detects an error.
            /// </returns>
            public Single ThermocoupleTemperature
            {
                get
                {
                    return IsSensorDataValid
                        ? (float) ((_sample[0] << 8 | _sample[1])*0.0625)
                        : (float) Double.NegativeInfinity;
                }
            }

            /// <summary>
            ///  Internal temperature  of the IC in degrees Celsius
            /// </summary>
            /// <returns>
            /// The temperature of the internal temperature of the MAX31855 IC on the ThermoClick, or Negative Infinity if the ThermoClick detects an error.
            /// </returns>
            /// <remarks>
            /// This could be useful for measuring ambient temperature inside an enclosure that the ThermoClick is mounted inside of.
            /// </remarks>
            internal Single InternalIcTemperature
            {
                get
                {
                    return IsSensorDataValid
                        ? (float) ((_sample[2] << 9 | (_sample[3] & 0xF8) << 1)*0.0015625)
                        : (float) Double.NegativeInfinity;
                }
            }

            //public override string ToString()
            //{
            //    string b0 = ((_sample[0] & 0x80) != 0 ? "1" : "0") + ((_sample[0] & 0x40) != 0 ? "1" : "0") +
            //                ((_sample[0] & 0x20) != 0 ? "1" : "0") + ((_sample[0] & 0x10) != 0 ? "1" : "0") +
            //                ((_sample[0] & 0x08) != 0 ? "1" : "0") + ((_sample[0] & 0x04) != 0 ? "1" : "0") +
            //                ((_sample[0] & 0x02) != 0 ? "1" : "0") + ((_sample[0] & 0x01) != 0 ? "1" : "0");

            //    string b1 = ((_sample[1] & 0x80) != 0 ? "1" : "0") + ((_sample[1] & 0x40) != 0 ? "1" : "0") +
            //                ((_sample[1] & 0x20) != 0 ? "1" : "0") + ((_sample[1] & 0x10) != 0 ? "1" : "0") +
            //                ((_sample[1] & 0x08) != 0 ? "1" : "0") + ((_sample[1] & 0x04) != 0 ? "1" : "0") +
            //                ((_sample[1] & 0x02) != 0 ? "1" : "0") + ((_sample[1] & 0x01) != 0 ? "1" : "0");

            //    string b2 = ((_sample[2] & 0x80) != 0 ? "1" : "0") + ((_sample[2] & 0x40) != 0 ? "1" : "0") +
            //                ((_sample[2] & 0x20) != 0 ? "1" : "0") + ((_sample[2] & 0x10) != 0 ? "1" : "0") +
            //                ((_sample[2] & 0x08) != 0 ? "1" : "0") + ((_sample[2] & 0x04) != 0 ? "1" : "0") +
            //                ((_sample[2] & 0x02) != 0 ? "1" : "0") + ((_sample[2] & 0x01) != 0 ? "1" : "0");

            //    string b3 = ((_sample[3] & 0x80) != 0 ? "1" : "0") + ((_sample[3] & 0x40) != 0 ? "1" : "0") +
            //                ((_sample[3] & 0x20) != 0 ? "1" : "0") + ((_sample[3] & 0x10) != 0 ? "1" : "0") +
            //                ((_sample[3] & 0x08) != 0 ? "1" : "0") + ((_sample[3] & 0x04) != 0 ? "1" : "0") +
            //                ((_sample[3] & 0x02) != 0 ? "1" : "0") + ((_sample[3] & 0x01) != 0 ? "1" : "0");

            //    return b0 + "-" + b1 + "-" + b2 + "-" + b3;
            //}
        }
    }
}